! This program solves the linearized equatorial beta-plane shallow water model 
! Discretization is based on Arakawa C-grid
! Boundaries are closed

   program shallow
   implicit none

! Defining constanst
     real :: g=9.8                    ! Gravitational acceleration (m/s^2)
     real :: beta=4*3.1416/86400/6e6  ! Gradient of coriolis parameter (1/s/m)

! Variables
   integer                           :: i,j,n,ix,jx,nx,n_euler,nout
   integer                           :: iunit=1
   real,allocatable,dimension(:,:,:) :: eta,u,v
   real                              :: dx, dy, dt, H, rayleigh
   real                              :: tgx, tgy, tHx, tHy, r
   real,allocatable,dimension(:)     :: tfu,tfv
   namelist /param/ dx, dy, dt, ix, jx, nx, H, rayleigh, n_euler, nout
   

! Read in the grid definition (the eta grid, that is)
! and open the output file
   open(11,file="shallow_beta.nml")
   read(11,param)
   close(11)
   open(iunit,file="shallow_beta.dat",form="unformatted",recordtype="stream")
   
! Allocate the variables' arrays in memory
   allocate(eta(ix,jx,-1:1),u(ix+1,jx,-1:1),v(ix,jx+1,-1:1))
   allocate(tfu(jx),tfv(jx+1))

! Print out some info

  print'("dx = ",f," km",4x,"dy = ",f," km",4x," dt =",f," s")',dx/1000,dy/1000,dt
  print'("ix = ",i4,4x,"jx = ",i4,4x,"nx = ",i4)',ix,jx,nx
  print*,""
  print'("C=",f," m/s (Kelvin wave speed)")',sqrt(g*H)
  print'("Rd=",f," km (Deformation radius)")',sqrt(sqrt(g*H)/beta)/1000
  print*,""
  print'("CFL = ",f," (should be < 0.3)")',sqrt(g*H)*dt/min(dx,dy)
  print*,""
  print'("Output every ",f," s (total of ",i," times)")',nout*dt,int(nx/nout)
  print*,""

! Initialize the fields
   
   u=0.
   v=0.
   do j=1,jx
      do i=1,ix
         ! Gaussian blob on the equator
         eta(i,j,-1)=exp(-((j-jx/2.)/20.)**2-((i-ix/2.)/20)**2)
      end do
   end do

! Define some values for computational use
! all the values were defined with respect to the centered time 
! discretization scheme.
! tfu and tfv depend on the Arakawa's C-grid
   
   do j=1,jx
     tfu(j) = dt/2.*beta*dy*(j-jx/2.)  ! units s^2/m
   end do
   do j=1,jx+1
     tfv(j) = dt/2.*beta*dy*(j-(jx+1)/2.)  ! units s^2/m
   end do
   tgx = 2*dt*g/dx       !units 1/s
   tgy = 2*dt*g/dy       !units 1/s
   r   = 2*dt*rayleigh   !adimensional units []
   tHx = 2*dt*H/dx       !units s
   tHy = 2*dt*H/dy       !units s

! Using euler scheme, calculate the fields for the 
! first time step (needed for integrating with leap-frog)
   eta(:,:,0)=eta(:,:,-1)
   call output(eta(:,:,0),u(:,:,0),v(:,:,0),ix,jx,iunit)
   do n=1,n_euler
      do j=1,jx
         do i=2,ix
            u(i,j,1)=u(i,j,0) + (0.5*tfu(j)/float(n_euler))*(v(i-1,j,0)     &
  &                  +v(i,j,0)+v(i-1,j+1,0)+v(i,j+1,0))                 &
  &                  -(0.5*tgx/float(n_euler))*(eta(i,j,0)-eta(i-1,j,0))
        end do
      end do
      do j=2,jx
         do i=1,ix
            v(i,j,1)=v(i,j,0) - (0.5*tfv(j)/float(n_euler))*(u(i,j-1,0)     &
  &                  +u(i,j,0)+u(i+1,j-1,0)+u(i+1,j,0))                &
  &                  -(0.5*tgy/float(n_euler))*(eta(i,j,0)-eta(i,j-1,0))
         end do
      end do
      do j=1,jx
         do i=1,ix 
            eta(i,j,1)=eta(i,j,0) - (0.5*tHx/float(n_euler))*        &
  &                   (u(i+1,j,0)-u(i,j,0))-(0.5*tHy/float(n_euler)) &
  &                  *(v(i,j+1,0)-v(i,j,0))
         end do
      end do
      eta(:,:,0)=eta(:,:,1) 
      u(:,:,0)=u(:,:,1)
      v(:,:,0)=v(:,:,1)
   end do
   if (nout.eq.1)  call output(eta(:,:,0),u(:,:,0),v(:,:,0),ix,jx,iunit)

! Integrate using leap-frog scheme
   do n=3,nx 
      do j=1,jx
         do i=2,ix
            u(i,j,1) = u(i,j,-1) + tfu(j)*(v(i-1,j,0)+v(i,j,0)    &
  &                                +v(i-1,j+1,0)+v(i,j+1,0))  &
  &                           -tgx*(eta(i,j,0)-eta(i-1,j,0))  &
  &                           -r*u(i,j,0)
         end do
      end do
      do j=2,jx
         do i=1,ix
            v(i,j,1) = v(i,j,-1) - tfv(j)*(u(i,j-1,0)+u(i,j,0)    &
  &                                +u(i+1,j-1,0)+u(i+1,j,0))  &
  &                           -tgy*(eta(i,j,0)-eta(i,j-1,0)) &
  &                           -r*v(i,j,0)
         end do
      end do
      do j=1,jx
         do i=1,ix 
            eta(i,j,1) = eta(i,j,-1) - tHx*(u(i+1,j,0)-u(i,j,0)) &
  &                                  - tHy*(v(i,j+1,0)-v(i,j,0)) &
  &                           -r*eta(i,j,0)
         end do
      end do
      eta(:,:,-1)=eta(:,:,0) 
      u(:,:,-1)=u(:,:,0)
      v(:,:,-1)=v(:,:,0)
      eta(:,:,0)=eta(:,:,1) 
      u(:,:,0)=u(:,:,1)
      v(:,:,0)=v(:,:,1)
      if (mod(n-1,nout).eq.0) call output(eta(:,:,0),u(:,:,0),v(:,:,0),ix,jx,iunit)
   end do

   print*,"Done."
   print*,""

   end




   subroutine output(eta,u,v,ix,jx,iunit)
   implicit none
   integer                  :: i, j, ix, jx, iunit
   real, dimension(ix,jx)   :: eta,ueta,veta
   real, dimension(ix+1,jx) :: u
   real, dimension(ix,jx+1) :: v

   do j=1,jx
      do i=1,ix
         ueta(i,j)=(u(i,j)+u(i+1,j))/2.
         veta(i,j)=(v(i,j)+v(i,j+1))/2.
      end do
   end do

   write(iunit) eta
   write(iunit) ueta
   write(iunit) veta

   end
